@@ -18,14 +18,10 @@
 
 using namespace KSyntaxHighlighting;
 
-ThemeData *ThemeData::get(const Theme &theme)
-{
-    return theme.m_data.data();
-}
-
 ThemeData::ThemeData()
 {
     memset(m_editorColors, 0, sizeof(m_editorColors));
+    m_textStyles.resize(QMetaEnum::fromType<Theme::TextStyle>().keyCount());
 }
 
 /**
@@ -81,14 +77,26 @@ static inline TextStyleData readThemeData(const QJsonObject &obj)
 
 bool ThemeData::load(const QString &filePath)
 {
+    // flag first as done for the error cases
+    m_completelyLoaded = true;
+
     QFile loadFile(filePath);
     if (!loadFile.open(QIODevice::ReadOnly)) {
         return false;
     }
     const QByteArray jsonData = loadFile.readAll();
+    // look for metadata object
+    int metaDataStart = jsonData.indexOf("\"metadata\"");
+    int start = jsonData.indexOf('{', metaDataStart);
+    int end = jsonData.indexOf("}", metaDataStart);
+    if (start < 0 || end < 0) {
+        qCWarning(Log) << "Failed to parse theme file" << filePath << ":"
+                       << "no metadata object found";
+        return false;
+    }
 
     QJsonParseError parseError;
-    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonData, &parseError);
+    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonData.mid(start, (end + 1) - start), &parseError);
     if (parseError.error != QJsonParseError::NoError) {
         qCWarning(Log) << "Failed to parse theme file" << filePath << ":" << parseError.errorString();
         return false;
@@ -96,17 +104,39 @@ bool ThemeData::load(const QString &filePath)
 
     m_filePath = filePath;
 
-    QJsonObject obj = jsonDoc.object();
+    // we need more data later
+    m_completelyLoaded = false;
 
     // read metadata
-    const QJsonObject metadata = obj.value(QLatin1String("metadata")).toObject();
+    QJsonObject metadata = jsonDoc.object();
     m_name = metadata.value(QLatin1String("name")).toString();
     m_revision = metadata.value(QLatin1String("revision")).toInt();
+    return true;
+}
 
+void ThemeData::loadComplete()
+{
+    if (m_completelyLoaded) {
+        return;
+    }
+    m_completelyLoaded = true;
+
+    QFile loadFile(m_filePath);
+    if (!loadFile.open(QIODevice::ReadOnly)) {
+        return;
+    }
+    const QByteArray jsonData = loadFile.readAll();
+
+    QJsonParseError parseError;
+    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonData, &parseError);
+    if (parseError.error != QJsonParseError::NoError) {
+        qCWarning(Log) << "Failed to parse theme file" << m_filePath << ":" << parseError.errorString();
+        return;
+    }
+
+    QJsonObject obj = jsonDoc.object();
     // read text styles
-    static const auto styleIdx = Theme::staticMetaObject.indexOfEnumerator("TextStyle");
-    Q_ASSERT(styleIdx >= 0);
-    const auto metaEnumStyle = Theme::staticMetaObject.enumerator(styleIdx);
+    const auto metaEnumStyle = QMetaEnum::fromType<Theme::TextStyle>();
     const QJsonObject textStyles = obj.value(QLatin1String("text-styles")).toObject();
     for (int i = 0; i < metaEnumStyle.keyCount(); ++i) {
         Q_ASSERT(i == metaEnumStyle.value(i));
@@ -114,9 +144,7 @@ bool ThemeData::load(const QString &filePath)
     }
 
     // read editor colors
-    static const auto colorIdx = Theme::staticMetaObject.indexOfEnumerator("EditorColorRole");
-    Q_ASSERT(colorIdx >= 0);
-    const auto metaEnumColor = Theme::staticMetaObject.enumerator(colorIdx);
+    const auto metaEnumColor = QMetaEnum::fromType<Theme::EditorColorRole>();
     const QJsonObject editorColors = obj.value(QLatin1String("editor-colors")).toObject();
     for (int i = 0; i < metaEnumColor.keyCount(); ++i) {
         Q_ASSERT(i == metaEnumColor.value(i));
@@ -160,11 +188,12 @@ bool ThemeData::load(const QString &filePath)
     for (auto it = customStyles.begin(); it != customStyles.end(); ++it) {
         const auto obj = it.value().toObject();
         auto &overrideStyle = m_textStyleOverrides[it.key()];
-        for (auto it2 = obj.begin(); it2 != obj.end(); ++it2)
+        for (auto it2 = obj.begin(); it2 != obj.end(); ++it2) {
             overrideStyle.insert(it2.key(), readThemeData(it2.value().toObject()));
+        }
     }
 
-    return true;
+    return;
 }
 
 QString ThemeData::name() const
@@ -187,61 +216,71 @@ QString ThemeData::filePath() const
     return m_filePath;
 }
 
+TextStyleData ThemeData::textStyle(Theme::TextStyle style) const
+{
+    if (!m_completelyLoaded) {
+        const_cast<ThemeData *>(this)->loadComplete();
+    }
+    return m_textStyles[style];
+}
+
 QRgb ThemeData::textColor(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].textColor;
+    return textStyle(style).textColor;
 }
 
 QRgb ThemeData::selectedTextColor(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].selectedTextColor;
+    return textStyle(style).selectedTextColor;
 }
 
 QRgb ThemeData::backgroundColor(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].backgroundColor;
+    return textStyle(style).backgroundColor;
 }
 
 QRgb ThemeData::selectedBackgroundColor(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].selectedBackgroundColor;
+    return textStyle(style).selectedBackgroundColor;
 }
 
 bool ThemeData::isBold(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].bold;
+    return textStyle(style).bold;
 }
 
 bool ThemeData::isItalic(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].italic;
+    return textStyle(style).italic;
 }
 
 bool ThemeData::isUnderline(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].underline;
+    return textStyle(style).underline;
 }
 
 bool ThemeData::isStrikeThrough(Theme::TextStyle style) const
 {
-    Q_ASSERT(static_cast<int>(style) >= 0 && static_cast<int>(style) <= static_cast<int>(Theme::Others));
-    return m_textStyles[style].strikeThrough;
+    return textStyle(style).strikeThrough;
 }
 
 QRgb ThemeData::editorColor(Theme::EditorColorRole role) const
 {
+    if (!m_completelyLoaded) {
+        const_cast<ThemeData *>(this)->loadComplete();
+    }
     Q_ASSERT(static_cast<int>(role) >= 0 && static_cast<int>(role) <= static_cast<int>(Theme::TemplateReadOnlyPlaceholder));
     return m_editorColors[role];
 }
 
 TextStyleData ThemeData::textStyleOverride(const QString &definitionName, const QString &attributeName) const
 {
-    return m_textStyleOverrides.value(definitionName).value(attributeName);
+    if (!m_completelyLoaded) {
+        const_cast<ThemeData *>(this)->loadComplete();
+    }
+    auto it = m_textStyleOverrides.find(definitionName);
+    if (it != m_textStyleOverrides.end()) {
+        return it->value(attributeName);
+    }
+    return TextStyleData();
 }
